# -*- shell-script -*-
#
# bashini: functions for handling ini files in bash
# Copyright (C) 2015 Heinz Deinhart
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

# bashini version 0.2

function bashini_fatal() {
    echo "bashini_fatal: $*" > /dev/stderr
    exit 1
}

# usage: read <db-name> <inputfile>
function bashini_read() {
    local ininame=$1
    local inputfile=$2

    unset bashini_db_$ininame
    declare -g -A bashini_db_$ininame

    local linenr=0
    local inidb=()
    local section=

    local extglobsave=$(shopt -p extglob)
    shopt -s extglob

    while read -r line || [[ -n "$line" ]]
    do
	local linenr=$((linenr + 1))

        # ignore empty lines and comments
	[[ "$line" =~ ^([[:space:]]*|[\;\#].*)$ ]] && {
	    continue
	}

	# sections
	[[ "$line" =~ ^\[([[:alnum:]_\-]+)\][[:space:]]*$ ]] && {
	    section=${BASH_REMATCH[1]}
	    eval bashini_db_$ininame[\"$section§\"]=\"\"
	    continue
	}

	# values
	[[ "$line" =~ ^[[:space:]]*([[:alnum:]]([[:alnum:] _\-]*[[:alnum:]_\-])?)[[:space:]]*=[[:space:]]*(.*)[[:space:]]*$ ]] && {
	    eval bashini_db_$ininame[\"$section§${BASH_REMATCH[1]}\"]=\"\${BASH_REMATCH[3]}\"
	    continue
	}

	echo "parse error at line $linenr near \"$line\""
	eval "$extglobsave"
	return 1
    done < $inputfile

    eval "$extglobsave"
    return 0
}

# usage: ensure_db <db-name>
function bashini_ensure_db() {
    local ininame=$1

    eval [[ -z "\${bashini_db_${ininame}[@]+x}" ]] && {
	echo "bashini: no such db: $ininame" > /dev/stderr
	return 1
    }
    return 0
}

# usage: key_exists <db-name> <key> [<section>]
#  if no section is specified, global context is searched
function bashini_key_exists() {
    [[ $# == 2 || $# == 3 ]] || \
	bashini_fatal "key_exists needs 2 or 3 args but called with $#"

    local ininame=$1
    local key=$2
    local section=""
    [[ $# == 3 ]] && section=$3

    eval [[ -z "\${bashini_db_${ininame}[\"$section§$key\"]+x}" ]] && return 1
    return 0
}

# usage: section_exists <db-name> <section>
#  if no section is specified, global context is searched
function bashini_section_exists() {
    [[ $# == 2 ]] || \
	bashini_fatal "section_exists needs 2 args but called with $#"

    local ininame=$1
    local section=$2

    eval [[ -z "\${bashini_db_${ininame}[\"$section§\"]+x}" ]] && return 1
    return 0
}

# usage: get_value <variablename> <db-name> <key> <section>
function bashini_get_value() {
    [[ $# == 3 || $# == 4 ]] || \
	bashini_fatal "get_value needs 3 or 4 args but called with $#"

    local retvarname=$1
    local ininame=$2
    local key=$3
    local section=""
    [[ $# == 4 ]] && section=$4

    bashini_ensure_db $ininame || return 1

    bashini_key_exists $ininame "$key" "$section" || return 1

    eval printf -v \$retvarname \"\${bashini_db_$ininame[\$section§\$key]}\"

    return 0
}

# get a key from section and take default from global context if not found
# usage get_value_with_default <variablename> <db-name> <key> <section>
function bashini_get_value_with_default() {
    [[ $# == 4 ]] || \
	bashini_fatal "get_value_with_default needs 4 args but called with $#"

    local retvarname=$1
    local ininame=$2
    local key=$3
    local section=$4

    bashini_get_value $retvarname $ininame "$key" "$section" || \
	bashini_get_value $retvarname $ininame "$key"
}
